﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Diagnostics;
using System.Linq;
using CompanyDatabaseInitialization;
using DataLayer;
using DomainClasses;
using Microsoft.VisualStudio.TestTools.UnitTesting;


namespace AutomatedTests
{
  [TestClass]
  public class TestContexts
  {
    public TestContexts()
    {
      HibernatingRhinos.Profiler.Appender.EntityFramework.EntityFrameworkProfiler.Initialize();
      Database.SetInitializer<ShippingContext>(null);
    }

      //RUN THIS TEST FIRST TO CREATE THE DATABASE
      //IT WILL ALSO FORCE THE SEED METHOD WHICH WILL CREATE
      //SOME DATA AND A NEEDED DATABASE VIEW
      //SEE THE COMPANYDATABASEMODEL.cs FILE FOR DETAILS
      [TestMethod]
      public void InitializeDatabase()
      {
          Database.SetInitializer(new DemoDBInitializer());
          using (var context=new CompanyDatabase())
          {
              context.Database.Initialize(force:true);
          }
          
      }
  
    [TestMethod]
    public void CanRetrieveLineItemsToBeShipped()
    {
      using (var context = new ShippingContext())
      {
        Debug.WriteLine(context.Database.Connection.ConnectionString);
        Assert.IsTrue(context.ItemsToBeShipped.Any());
      }
    }
    [TestMethod]
    public void GetOrdersWithItemsToBeShipped()
    {
      using (var context = new ShippingContext())
      {//var orders=context.ItemsToBeShipped.Select(i=>i.OrderShippingDetails)

        var orderQuery = context.Orders
            .Include(o => o.Customer.ShippingAddress)
            .Where(o => o.OpenLineItems.Any());
        var orders = orderQuery.ToList();
        Assert.IsTrue(orders.Any());
      }
    }

    [TestMethod]
      public void NewlyShippedItemNotInToBeShippedList()
      {
        int unshippedItemId;
        using (var context = new ShippingContext())
        {
          var unshippedItem = context.ItemsToBeShipped.First();
          unshippedItemId = unshippedItem.LineItemId;
          var shipment = new Shipment
                           {
                             DateShipped = DateTime.Now,
                             EntireOrder = true,
                             OrderId = unshippedItem.OrderId,
                             ShipperId = 1
                           };
          //new shipment won't have shipmentid so need navigation property to do an insert
          var newlyshippedItem = new ShippedLineItem
          {LineItemId = unshippedItemId, 
            Shipment = shipment};
          context.Shipments.Add(shipment);
          context.Entry(newlyshippedItem).State=EntityState.Modified;
          context.SaveChanges();
        }
          using (var context = new ShippingContext())
          {
            Assert.IsNull(context.ItemsToBeShipped.Find(unshippedItemId));
          }

      }

    }

    

 
  }
