using System.Data.Entity;
using DataLayer;
using DataLayer.Mapping;
using DomainClasses;
using System.Collections.Generic;
using System;

namespace CompanyDatabaseInitialization
{
    public class CompanyDatabase : DbContext
    {
        public CompanyDatabase()
            : base("DPSalesDatabase")
        { }


        public DbSet<Customer> Customers { get; set; }
        public DbSet<Employee> Employees { get; set; }
        public DbSet<SalaryHistory> SalaryHistories { get; set; }
        public DbSet<Order> Orders { get; set; }
        public DbSet<LineItem> LineItems { get; set; }
        public DbSet<Product> Products { get; set; }
        public DbSet<Shipment> Shipments { get; set; }
        public DbSet<Shipper> Shippers { get; set; }
        public DbSet<ShippingAddress> ShippingAddresses { get; set; }
        public DbSet<Payment> Payments { get; set; }
        public DbSet<Category> Categories { get; set; }
        public DbSet<Promotion> Promotions { get; set; }
        public DbSet<Return> Returns { get; set; }

        protected override void OnModelCreating(DbModelBuilder modelBuilder)
        {
            modelBuilder.Configurations.Add(new LineItemMap());
            modelBuilder.Configurations.Add(new ShippingAddressMap());
            modelBuilder.Entity<Product>().Ignore(t => t.LineItems);
        }
    }

    public class DemoDBInitializer : CreateDatabaseIfNotExists<CompanyDatabase>
    {
        protected override void Seed(CompanyDatabase context)
        {
            var biking = new Category { Name = "Biking" };
            var blackbike = new Product { Name = "Black Road Bike", ProductNumber = "BRB1", Category = biking, Color = ProductColor.Black, SellStartDate = DateTime.Now };
            var redbike = new Product { Name = "Red Road Bike", ProductNumber = "RRB1", Category = biking, Color = ProductColor.Red, SellStartDate = DateTime.Now };
            var julie = new Customer
            {
                FirstName = "Julie",
                LastName = "Lerman",
                Orders = new List<Order>{ 
                 new Order{ OrderDate=DateTime.Now, ModifiedDate=DateTime.Now,
                     LineItems=new List<LineItem>{
                         new LineItem{Product=blackbike, OrderQty=1}
                     }
                 }
             }
            };
            var boris = new Customer
            {
                FirstName = "Boris",
                LastName = "Badenov",
                Orders = new List<Order>{ 
                 new Order{ OrderDate=DateTime.Now, ModifiedDate=DateTime.Now,
                     LineItems=new List<LineItem>{
                         new LineItem{Product=redbike, OrderQty=1}
                     }
                 }}
            };
            var natasha = new Customer { FirstName = "Natasha", LastName = "Fatale" };
            context.Customers.Add(julie);
            context.Customers.Add(boris);
            context.Customers.Add(natasha);
            context.Shippers.Add(new Shipper{Name="No Bump Shipping" });
            context.Shippers.Add(new Shipper {Name="From Here to There Shippers"});
            string CreateView = "CREATE VIEW ItemsToBeShipped " +
"AS " +
"SELECT L.LineItemId,L.OrderId,L.OrderQty as Quantity,P.Name as ProductName, p.ProductNumber " +
"FROM lineitems L,products P  " +
"WHERE L.Productid=p.productid and (shipmentid=0 or shipmentid is null)";

            context.Database.ExecuteSqlCommand(CreateView);
            base.Seed(context);
          
        }
    }
}

